@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *)
- 
+
 open Ast
 open Type
 open Common
@@ -63,7 +63,7 @@ let extend_remoting ctx c t p async prot =
 	let new_name = (if async then "Async_" else "Remoting_") ^ t.tname in
 	(* check if the proxy already exists *)
 	let t = (try
-		Typeload.load_type_def ctx p (fst path,new_name)
+		Typeload.load_type_def ctx p { tpackage = fst path; tname = new_name; tparams = []; tsub = None }
 	with
 		Error (Module_not_found _,p2) when p == p2 ->
 	(* build it *)
@@ -71,10 +71,10 @@ let extend_remoting ctx c t p async prot =
 	let decls = (try Typeload.parse_module ctx path p with e -> ctx.com.package_rules <- rules; raise e) in
 	ctx.com.package_rules <- rules;
 	let base_fields = [
-		(FVar ("__cnx",None,[],Some (TPNormal { tpackage = ["haxe";"remoting"]; tname = if async then "AsyncConnection" else "Connection"; tparams = [] }),None),p);
+		(FVar ("__cnx",None,[],Some (TPNormal { tpackage = ["haxe";"remoting"]; tname = if async then "AsyncConnection" else "Connection"; tparams = []; tsub = None }),None),p);
 		(FFun ("new",None,[APublic],[],{ f_args = ["c",false,None,None]; f_type = None; f_expr = (EBinop (OpAssign,(EConst (Ident "__cnx"),p),(EConst (Ident "c"),p)),p) }),p);
 	] in
-	let tvoid = TPNormal { tpackage = []; tname = "Void"; tparams = [] } in
+	let tvoid = TPNormal { tpackage = []; tname = "Void"; tparams = []; tsub = None } in
 	let build_field is_public acc (f,p) =
 		match f with
 		| FFun ("new",_,_,_,_) ->
@@ -86,11 +86,11 @@ let extend_remoting ctx c t p async prot =
 			let fargs, eargs = if async then match ftype with
 				| Some tret -> f.f_args @ ["__callb",true,Some (TPFunction ([tret],tvoid)),None], eargs @ [EConst (Ident "__callb"),p]
 				| _ -> f.f_args, eargs @ [EConst (Ident "null"),p]
-			else 
+			else
 				f.f_args, eargs
 			in
 			let id = (EConst (String name), p) in
-			let id = if prot then id else ECall ((EConst (Ident "__unprotect__"),p),[id]),p in 
+			let id = if prot then id else ECall ((EConst (Ident "__unprotect__"),p),[id]),p in
 			let expr = ECall (
 				(EField (
 					(ECall ((EField ((EConst (Ident "__cnx"),p),"resolve"),p),[id]),p),
@@ -110,12 +110,12 @@ let extend_remoting ctx c t p async prot =
 		match d with
 		| EClass c, p when c.d_name = t.tname ->
 			let is_public = List.mem HExtern c.d_flags || List.mem HInterface c.d_flags in
-			let fields = List.rev (List.fold_left (build_field is_public) base_fields c.d_data) in	
+			let fields = List.rev (List.fold_left (build_field is_public) base_fields c.d_data) in
 			(EClass { c with d_flags = []; d_name = new_name; d_data = fields },p)
 		| _ -> d
 	) decls in
 	let m = Typeload.type_module ctx (t.tpackage,new_name) decls p in
-	try 
+	try
 		List.find (fun tdecl -> snd (t_path tdecl) = new_name) m.mtypes
 	with Not_found ->
 		error ("Module " ^ s_type_path path ^ " does not define type " ^ t.tname) p
@@ -152,15 +152,14 @@ let build_generic ctx c p tl =
 	if !recurse then
 		TInst (c,tl) (* build a normal instance *)
 	else try
-		Typeload.load_normal_type ctx { tpackage = pack; tname = name; tparams = [] } p false
+		Typeload.load_instance ctx { tpackage = pack; tname = name; tparams = []; tsub = None } p false
 	with Error(Module_not_found path,_) when path = (pack,name) ->
 		let m = (try Hashtbl.find ctx.modules (Hashtbl.find ctx.types_module c.cl_path) with Not_found -> assert false) in
 		let ctx = { ctx with local_types = m.mtypes @ ctx.local_types } in
 		let cg = mk_class (pack,name) c.cl_pos None false in
 		let mg = {
 			mpath = cg.cl_path;
-			mtypes = [TClassDecl cg];
-			mimports = [];
+			mtypes = [TClassDecl cg];			
 		} in
 		Hashtbl.add ctx.modules mg.mpath mg;
 		let rec loop l1 l2 =
@@ -188,8 +187,13 @@ let build_generic ctx c p tl =
 		if c.cl_super <> None || c.cl_init <> None || c.cl_dynamic <> None then error "This class can't be generic" p;
 		if c.cl_ordered_statics <> [] then error "A generic class can't have static fields" p;
 		cg.cl_kind <- KGenericInstance (c,tl);
+		cg.cl_interface <- c.cl_interface;
 		cg.cl_constructor <- (match c.cl_constructor with None -> None | Some c -> Some (build_field c));
-		cg.cl_implements <- List.map (fun (i,tl) -> i, List.map build_type tl) c.cl_implements;
+		cg.cl_implements <- List.map (fun (i,tl) ->
+			(match build_type (TInst (i, List.map build_type tl)) with
+			| TInst (i,tl) -> i, tl
+			| _ -> assert false)
+		) c.cl_implements;
 		cg.cl_ordered_fields <- List.map (fun f ->
 			let f = build_field f in
 			cg.cl_fields <- PMap.add f.cf_name f cg.cl_fields;
@@ -201,7 +205,7 @@ let build_generic ctx c p tl =
 (* HAXE.XML.PROXY *)
 
 let extend_xml_proxy ctx c t file p =
-	let t = Typeload.load_type ctx p t in
+	let t = Typeload.load_complex_type ctx p t in
 	let file = (try Common.find_file ctx.com file with Not_found -> file) in
 	let used = ref PMap.empty in
 	let rec delay() =
@@ -318,9 +322,9 @@ let rec local_usage f e =
 			List.iter (fun (n,_,t) -> f (Declare (n,t))) tf.tf_args;
 			local_usage f tf.tf_expr;
 		in
-		f (Function cc)		
+		f (Function cc)
 	| TBlock l ->
-		f (Block (fun f -> List.iter (local_usage f) l))		
+		f (Block (fun f -> List.iter (local_usage f) l))
 	| TFor (v,t,it,e) ->
 		local_usage f it;
 		f (Loop (fun f ->
@@ -342,7 +346,7 @@ let rec local_usage f e =
 	| TMatch (e,_,cases,def) ->
 		local_usage f e;
 		List.iter (fun (_,vars,e) ->
-			let cc f = 
+			let cc f =
 				(match vars with
 				| None -> ()
 				| Some l ->	List.iter (fun (vo,t) -> match vo with None -> () | Some v -> f (Declare (v,t))) l);
@@ -362,7 +366,7 @@ let rec local_usage f e =
 	by value. It transforms the following expression :
 
 	for( x in array )
-		funs.push(function() return x++);	
+		funs.push(function() return x++);
 
 	Into the following :
 
@@ -391,17 +395,17 @@ let block_vars ctx e =
 	and wrap used e =
 		match e.eexpr with
 		| TVars vl ->
-			let vl = List.map (fun (v,vt,e) ->
+			let vl = List.map (fun (v,vt,ve) ->
 				if PMap.mem v used then begin
-					let vt = t.tarray vt in					
-					v, vt, (match e with None -> None | Some e -> Some (mk (TArrayDecl [wrap used e]) (t.tarray e.etype) e.epos))
+					let vt = t.tarray vt in
+					v, vt, Some (mk (TArrayDecl (match ve with None -> [] | Some e -> [wrap used e])) vt e.epos)
 				end else
-					v, vt, (match e with None -> None | Some e -> Some (wrap used e))
+					v, vt, (match ve with None -> None | Some e -> Some (wrap used e))
 			) vl in
 			{ e with eexpr = TVars vl }
 		| TLocal v when PMap.mem v used ->
 			mk (TArray ({ e with etype = t.tarray e.etype },mk (TConst (TInt 0l)) t.tint e.epos)) e.etype e.epos
-		| TFor (v,vt,it,expr) when PMap.mem v used ->			
+		| TFor (v,vt,it,expr) when PMap.mem v used ->
 			let vtmp = gen_unique() in
 			let it = wrap used it in
 			let expr = wrap used expr in
@@ -410,7 +414,7 @@ let block_vars ctx e =
 			let catchs = List.map (fun (v,t,e) ->
 				let e = wrap used e in
 				if PMap.mem v used then
-					let vtmp = gen_unique()	in				
+					let vtmp = gen_unique()	in
 					vtmp, t, concat (mk_init v t vtmp e.epos) e
 				else
 					v, t, e
@@ -437,7 +441,7 @@ let block_vars ctx e =
 			let def = match def with None -> None | Some e -> Some (wrap used e) in
 			mk (TMatch (wrap used expr,enum,cases,def)) e.etype e.epos
 		| TFunction f ->
-			(* 
+			(*
 				list variables that are marked as used, but also used in that
 				function and which are not declared inside it !
 			*)
@@ -445,15 +449,15 @@ let block_vars ctx e =
 			let tmp_used = ref (PMap.foldi PMap.add used PMap.empty) in
 			let rec browse = function
 				| Block f | Loop f | Function f -> f browse
-				| Use v -> 
-					(try 
+				| Use v ->
+					(try
 						fused := PMap.add v (PMap.find v !tmp_used) !fused;
 					with Not_found ->
 						())
 				| Declare (v,_) ->
 					tmp_used := PMap.remove v !tmp_used
 			in
-			local_usage browse e;			
+			local_usage browse e;
 			let vars = PMap.foldi (fun v vt acc -> (v,t.tarray vt) :: acc) !fused [] in
 			(* in case the variable has been marked as used in a parallel scope... *)
 			let fexpr = ref (wrap used f.tf_expr) in
@@ -572,7 +576,7 @@ let check_local_vars_init e =
 			let init = (try PMap.find name !vars with Not_found -> true) in
 			if not init then error ("Local variable " ^ name ^ " used without being initialized") e.epos;
 		| TVars vl ->
-			List.iter (fun (v,_,eo) -> 
+			List.iter (fun (v,_,eo) ->
 				let init = (match eo with None -> false | Some e -> loop vars e; true) in
 				declared := v :: !declared;
 				vars := PMap.add v init !vars
@@ -627,7 +631,7 @@ let check_local_vars_init e =
 				vars := old;
 				v
 			) catches in
-			loop vars e;		
+			loop vars e;
 			join vars cvars;
 		| TSwitch (e,cases,def) ->
 			loop vars e;
@@ -675,8 +679,10 @@ let check_local_vars_init e =
 (* -------------------------------------------------------------------------- *)
 (* POST PROCESS *)
 
-let post_process ctx filters =
-	List.iter (function
+let post_process ctx filters tfilters =
+	List.iter (fun t ->
+		List.iter (fun f -> f t) tfilters;
+		match t with
 		| TClassDecl c ->
 			let process_field f =
 				match f.cf_expr with
@@ -691,7 +697,7 @@ let post_process ctx filters =
 			| Some f -> process_field f);
 			(match c.cl_init with
 			| None -> ()
-			| Some e ->				
+			| Some e ->
 				c.cl_init <- Some (List.fold_left (fun e f -> f e) e filters));
 		| TEnumDecl _ -> ()
 		| TTypeDecl _ -> ()
@@ -717,7 +723,7 @@ let stack_context_init com stack_var exc_var pos_var tmp_var use_add p =
 	let t = com.type_api in
 	let st = t.tarray t.tstring in
 	let stack_e = mk (TLocal stack_var) st p in
-	let exc_e = mk (TLocal exc_var) st p in	
+	let exc_e = mk (TLocal exc_var) st p in
 	let stack_pop = fcall stack_e "pop" [] t.tstring p in
 	let stack_push c m =
 		fcall stack_e "push" [
@@ -767,7 +773,7 @@ let rec stack_block_loop ctx e =
 			ctx.stack_pop;
 			e;
 		]) e.etype e.epos
-	| TReturn (Some e) ->	
+	| TReturn (Some e) ->
 		ctx.stack_return (stack_block_loop ctx e)
 	| TTry (v,cases) ->
 		let v = stack_block_loop ctx v in
@@ -783,18 +789,73 @@ let rec stack_block_loop ctx e =
 	| _ ->
 		map_expr (stack_block_loop ctx) e
 
-let stack_block ctx c m e =	
+let stack_block ctx c m e =
 	match (mk_block e).eexpr with
-	| TBlock l -> 
+	| TBlock l ->
 		mk (TBlock (
 			ctx.stack_push c m ::
-			ctx.stack_save_pos :: 
+			ctx.stack_save_pos ::
 			List.map (stack_block_loop ctx) l
 			@ [ctx.stack_pop]
 		)) e.etype e.epos
 	| _ ->
 		assert false
 
+(* -------------------------------------------------------------------------- *)
+(* FIX OVERRIDES *)
+
+(*
+	on some platforms which doesn't support type parameters, we must have the
+	exact same type for overriden/implemented function as the original one
+*)
+let fix_override c f fd =
+	c.cl_fields <- PMap.remove f.cf_name c.cl_fields;
+	let rec find_field c interf =
+		try
+			(match c.cl_super with
+			| None ->
+				raise Not_found
+			| Some (c,_) ->
+				find_field c false)
+		with Not_found -> try
+			let rec loop = function
+				| [] ->
+					raise Not_found
+				| (c,_) :: l ->
+					try
+						find_field c true
+					with
+						Not_found -> loop l
+			in
+			loop c.cl_implements
+		with Not_found ->
+			interf, PMap.find f.cf_name c.cl_fields
+	in
+	let f2 = (try Some (find_field c true) with Not_found -> None) in
+	let f = (match f2 with
+		| Some (interf,f2) ->
+			let targs, tret = (match follow f2.cf_type with TFun (args,ret) -> args, ret | _ -> assert false) in
+			let fd2 = { fd with tf_args = List.map2 (fun (n,c,t) (_,_,t2) -> (n,c,t2)) fd.tf_args targs; tf_type = tret } in
+			let fde = (match f.cf_expr with None -> assert false | Some e -> e) in
+			{ f with cf_expr = Some { fde with eexpr = TFunction fd2 } }
+		| _ -> f
+	) in
+	c.cl_fields <- PMap.add f.cf_name f c.cl_fields;
+	f
+
+let fix_overrides com t =
+	match com.platform, t with
+	| Flash9, TClassDecl c ->
+		c.cl_ordered_fields <- List.map (fun f ->
+			match f.cf_expr with
+			| Some { eexpr = TFunction fd } when f.cf_set <> NormalAccess && f.cf_set <> MethodAccess true ->
+				fix_override c f fd
+			| _ ->
+				f
+		) c.cl_ordered_fields
+	| _ ->
+		()
+
 (* -------------------------------------------------------------------------- *)
 (* MISC FEATURES *)
 
@@ -884,10 +945,66 @@ let rec constructor_side_effects e =
 		true
 	| TBinop _ | TTry _ | TIf _ | TBlock _ | TVars _
 	| TFunction _ | TArrayDecl _ | TObjectDecl _
-	| TParenthesis _ | TTypeExpr _ | TEnumField _ | TLocal _ 
-	| TConst _ | TContinue | TBreak -> 
-		try 
+	| TParenthesis _ | TTypeExpr _ | TEnumField _ | TLocal _
+	| TConst _ | TContinue | TBreak ->
+		try
 			Type.iter (fun e -> if constructor_side_effects e then raise Exit) e;
 			false;
 		with Exit ->
 			true
+
+(*
+	Make a dump of the full typed AST of all types
+*)
+let dump_types com =
+	let s_type = s_type (Type.print_context()) in
+	let params = function [] -> "" | l -> Printf.sprintf "<%s>" (String.concat "," (List.map (fun (n,t) -> n ^ " : " ^ s_type t) l)) in
+	let rec create acc = function
+		| [] -> ()
+		| d :: l ->
+			let dir = String.concat "/" (List.rev (d :: acc)) in
+			if not (Sys.file_exists dir) then Unix.mkdir dir 0o755;
+			create (d :: acc) l
+	in
+	List.iter (fun mt ->
+		let path = Type.t_path mt in
+		let dir = "dump" :: fst path in
+		create [] dir;
+		let ch = open_out (String.concat "/" dir ^ "/" ^ snd path ^ ".dump") in
+		let buf = Buffer.create 0 in
+		let print fmt = Printf.kprintf (fun s -> Buffer.add_string buf s) fmt in
+		(match mt with
+		| Type.TClassDecl c ->
+			let print_field stat f =
+				print "\t%s%s%s%s" (if stat then "static " else "") (if f.cf_public then "public " else "") f.cf_name (params f.cf_params);
+				print "(%s,%s) : %s" (s_access f.cf_get) (s_access f.cf_set) (s_type f.cf_type);
+				(match f.cf_expr with
+				| None -> ()
+				| Some e -> print "\n\n\t = %s" (Type.s_expr s_type e));
+				print ";\n\n";
+			in
+			print "%s%s%s %s%s" (if c.cl_private then "private " else "") (if c.cl_extern then "extern " else "") (if c.cl_interface then "interface" else "class") (s_type_path path) (params c.cl_types);
+			(match c.cl_super with None -> () | Some (c,pl) -> print " extends %s" (s_type (TInst (c,pl))));
+			List.iter (fun (c,pl) -> print " implements %s" (s_type (TInst (c,pl)))) c.cl_implements;
+			(match c.cl_dynamic with None -> () | Some t -> print " implements Dynamic<%s>" (s_type t));
+			(match c.cl_array_access with None -> () | Some t -> print " implements ArrayAccess<%s>" (s_type t));
+			print "{\n";
+			(match c.cl_constructor with
+			| None -> ()
+			| Some f -> print_field false f);
+			List.iter (print_field false) c.cl_ordered_fields;
+			List.iter (print_field true) c.cl_ordered_statics;
+			print "}";
+		| Type.TEnumDecl e ->
+			print "%s%senum %s%s {\n" (if e.e_private then "private " else "") (if e.e_extern then "extern " else "") (s_type_path path) (params e.e_types);
+			List.iter (fun n ->
+				let f = PMap.find n e.e_constrs in
+				print "\t%s : %s;\n" f.ef_name (s_type f.ef_type);
+			) e.e_names;
+			print "}"
+		| Type.TTypeDecl t ->
+			print "%stype %s%s = %s" (if t.t_private then "private " else "") (s_type_path path) (params t.t_types) (s_type t.t_type);
+		);
+		output_string ch (Buffer.contents buf);
+		close_out ch
+	) com.types
