@@ -31,14 +31,26 @@ let type_inline ctx cf f ethis params tret p =
 	let pnames = List.map (fun (name,_,t) ->
 		let name = add_local ctx name t in
 		Hashtbl.add hcount name (ref 0);
-		name
+		(name,t)
 	) f.tf_args in
-	(* use default values for null arguments *)
-	let params = List.map2 (fun e (_,opt,t) ->
-		match e.eexpr, opt with
-		| TConst TNull , Some c -> mk (TConst c) t e.epos
-		| _ -> e
-	) params f.tf_args in
+	(* use default values for null/unset arguments *)
+	let rec loop pl al =
+		match pl, al with
+		| [], [] -> []
+		| e :: pl, (name, opt, t) :: al ->
+			if is_nullable t && is_null e.etype then Hashtbl.add lsets name (); (* force coerce *)
+			(match e.eexpr, opt with
+			| TConst TNull , Some c -> mk (TConst c) t e.epos
+			| _ -> e) :: loop pl al
+		| [], (_,opt,t) :: al ->
+			(match opt with
+			| None -> assert false
+			| Some c -> mk (TConst c) t p) :: loop [] al
+		| _ :: _, [] ->
+			assert false
+	in
+	let params = loop params f.tf_args in
+	let ethis = (match ethis.eexpr with TConst TSuper -> { ethis with eexpr = TConst TThis } | _ -> ethis) in
 	let vthis = gen_local ctx ethis.etype in
 	let this_count = ref 0 in
 	let local i =
@@ -51,6 +63,11 @@ let type_inline ctx cf f ethis params tret p =
 		| Some e -> Some (f e)
 	in
 	let has_vars = ref false in
+	(*
+		here, we try to eliminate final returns from the expression tree.
+		However, this is not entirely correct since we don't yet correctly propagate
+		the type of returned expressions upwards ("return" expr itself being Dynamic)
+	*)
 	let rec map term e =
 		let e = { e with epos = p } in
 		match e.eexpr with
@@ -74,25 +91,37 @@ let type_inline ctx cf f ethis params tret p =
 		| TFor (v,t,e1,e2) ->
 			{ e with eexpr = TFor (local v,t,map false e1,map false e2) }
 		| TMatch (e,en,cases,def) ->
-			let term = (match def with None -> false | Some _ -> term) in
+			let term, t = (match def with Some d when term -> true, d.etype | _ -> false, e.etype) in
 			let cases = List.map (fun (i,vl,e) ->
 				i, opt (List.map (fun (n,t) -> opt local n, t)) vl, map term e
 			) cases in
-			{ e with eexpr = TMatch (map false e,en,cases,opt (map term) def) }
+			{ e with eexpr = TMatch (map false e,en,cases,opt (map term) def); etype = t }
 		| TTry (e1,catches) ->
 			{ e with eexpr = TTry (map term e1,List.map (fun (v,t,e) -> local v,t,map term e) catches) }
 		| TBlock l ->
 			let old = save_locals ctx in
+			let t = ref e.etype in
 			let rec loop = function
+				| [] when term ->
+					t := mk_mono();
+					[mk (TConst TNull) (!t) p]
 				| [] -> []
-				| [e] -> [map term e]
+				| [e] ->
+					let e = map term e in
+					if term then t := e.etype;
+					[e]
 				| e :: l ->
 					let e = map false e in
 					e :: loop l
 			in
 			let l = loop l in
 			old();
-			{ e with eexpr = TBlock l }
+			{ e with eexpr = TBlock l; etype = !t }
+		| TIf (econd,eif,Some eelse) when term ->
+			let econd = map false econd in
+			let eif = map term eif in
+			let eelse = map term eelse in
+			{ e with eexpr = TIf(econd,eif,Some eelse); etype = eif.etype }
 		| TParenthesis _ | TIf (_,_,Some _) | TSwitch (_,_,Some _) ->
 			Type.map_expr (map term) e
 		| TUnop (op,pref,({ eexpr = TLocal s } as e1)) ->
@@ -116,15 +145,15 @@ let type_inline ctx cf f ethis params tret p =
 	locals();
 	let subst = ref PMap.empty in
 	Hashtbl.add hcount vthis this_count;
-	let vars = List.map2 (fun n e ->
+	let vars = List.map2 (fun (n,t) e ->
 		let flag = not (Hashtbl.mem lsets n) && (match e.eexpr with
 			| TLocal _ | TConst _ -> true
 			| _ ->
 				let used = !(Hashtbl.find hcount n) in
 				used <= 1
 		) in
-		(n,e.etype,e,flag)
-	) (vthis :: pnames) (ethis :: params) in
+		(n,t,e,flag)
+	) ((vthis,ethis.etype) :: pnames) (ethis :: params) in
 	let vars = List.fold_left (fun acc (n,t,e,flag) ->
 		if flag then begin
 			subst := PMap.add n e !subst;
@@ -142,7 +171,7 @@ let type_inline ctx cf f ethis params tret p =
 	let init = (match vars with [] -> None | l -> Some (mk (TVars (List.rev l)) ctx.api.tvoid p)) in
 	if Common.defined ctx.com "js" && (init <> None || !has_vars) then
 		None
-	else 
+	else
 		let e = (match e.eexpr, init with
 			| TBlock [e] , None -> { e with etype = tret; }
 			| _ , None -> { e with etype = tret; }
@@ -154,13 +183,13 @@ let type_inline ctx cf f ethis params tret p =
 		match cf.cf_params, tparams with
 		| [], ([],_) -> Some e
 		| _ ->
-			let pmonos = List.map (fun _ -> mk_mono()) cf.cf_params in			
+			let pmonos = List.map (fun _ -> mk_mono()) cf.cf_params in
 			let tmonos = snd tparams @ pmonos in
 			let tparams = fst tparams @ cf.cf_params in
 			let mt = apply_params tparams tmonos cf.cf_type in
 			unify_raise ctx mt (TFun (List.map (fun e -> "",false,e.etype) params,tret)) p;
 			(*
-				this is very expensive since we are building the substitution list for 
+				this is very expensive since we are building the substitution list for
 				every expression, but hopefully in such cases the expression size is small
 			*)
 			let map_type t = apply_params tparams tmonos t in
@@ -224,7 +253,7 @@ let optimize_for_loop ctx i e1 e2 p =
 					NormalWhile
 				)) t_void p;
 			])
-	| _ , TInst({ cl_path = [],"Array" },[pt]) 
+	| _ , TInst({ cl_path = [],"Array" },[pt])
 	| _ , TInst({ cl_path = ["flash"],"Vector" },[pt]) ->
 		let i = add_local ctx i pt in
 		let index = gen_local ctx t_int in
@@ -277,18 +306,13 @@ let optimize_for_loop ctx i e1 e2 p =
 (* ---------------------------------------------------------------------- *)
 (* REDUCE *)
 
-let rec reduce_loop com is_sub e =
+let rec reduce_loop ctx is_sub e =
 	let is_float t =
 		match follow t with
 		| TInst ({ cl_path = ([],"Float") },_) -> true
 		| _ -> false
 	in
-	let is_text_platform() =
-		match com.platform with
-		| Js | Php -> true 
-		| Neko | Flash | Flash9 | Cross | Cpp -> false
-	in
-	let e = Type.map_expr (reduce_loop com (match e.eexpr with TBlock _ -> false | _ -> true)) e in
+	let e = Type.map_expr (reduce_loop ctx (match e.eexpr with TBlock _ -> false | _ -> true)) e in
 	match e.eexpr with
 	| TIf ({ eexpr = TConst (TBool t) },e1,e2) ->
 		(if t then e1 else match e2 with None -> { e with eexpr = TBlock [] } | Some e -> e)
@@ -296,7 +320,7 @@ let rec reduce_loop com is_sub e =
 		(match flag with
 		| NormalWhile -> { e with eexpr = TBlock [] } (* erase sub *)
 		| DoWhile -> e) (* we cant remove while since sub can contain continue/break *)
-	| TBinop (op,e1,e2) ->		
+	| TBinop (op,e1,e2) ->
 		(match e1.eexpr, e2.eexpr with
 		| TConst (TInt 0l) , _ when op = OpAdd -> e2
 		| TConst (TInt 1l) , _ when op = OpMult -> e2
@@ -358,7 +382,7 @@ let rec reduce_loop com is_sub e =
 			| OpGte -> ebool ((>=) 0)
 			| OpLt -> ebool ((<) 0)
 			| OpLte -> ebool ((<=) 0)
-			| _ -> e)			
+			| _ -> e)
 		| TConst (TBool a), TConst (TBool b) ->
 			let ebool f =
 				{ e with eexpr = TConst (TBool (f a b)) }
@@ -379,6 +403,16 @@ let rec reduce_loop com is_sub e =
 			| OpBoolAnd when a  -> e1
 			| OpBoolOr when not a -> e1
 			| _ -> e)
+		| TEnumField (e1,f1), TEnumField (e2,f2) when e1 == e2 ->
+			(match op with
+			| OpEq -> { e with eexpr = TConst (TBool (f1 = f2)) }
+			| OpNotEq -> { e with eexpr = TConst (TBool (f1 <> f2)) }
+			| _ -> e)
+		| _, TCall ({ eexpr = TEnumField _ },_) | TCall ({ eexpr = TEnumField _ },_), _ ->
+			(match op with
+			| OpAssign -> e
+			| _ ->
+				error "You cannot directly compare enums with arguments. Use either 'switch' or 'Type.enumEq'" e.epos)
 		| _ -> e)
 	| TUnop (op,flag,esub) ->
 		(match op, esub.eexpr with
@@ -394,43 +428,57 @@ let rec reduce_loop com is_sub e =
 				e
 		| _ -> e
 		)
-	| TCall ({ eexpr = TFunction func },el) ->
-		let rec build term e =
-			match e.eexpr with
-			| TBlock el ->
-				(match List.rev el with
-				| [] -> e
-				| e1 :: el ->
-					let el = List.map (build false) (List.rev el) in
-					let e1 = build term e1 in
-					{ e with eexpr = TBlock (el @ [e1]) })
-			| TParenthesis _ | TIf (_,_,Some _) | TSwitch _ | TMatch _ | TTry _ ->
-				(* might only cause issues if some 'return' found in the first expression of if/switch/match *)
-				Type.map_expr (build term) e
-			| TReturn eo ->				
-				if not term then raise Exit;
-				(match eo with
-				| None -> { e with eexpr = TBlock [] }
-				| Some e -> build term e)
-			| _ ->
-				Type.map_expr (build false) e
-		in
-		(try
-			let body = build true func.tf_expr in
-			let body = (match body.eexpr with TBlock el -> el | _ -> [body]) in
-			let body = (match el with
-				| [] -> body
-				| _ ->					
-					if is_sub && is_text_platform() then raise Exit; 
-					mk (TVars (List.map2 (fun (p,_,t) e -> p,t,Some e) func.tf_args el)) com.type_api.tvoid e.epos :: body
-			) in
-			{ e with eexpr = TBlock body }
-		with
-			Exit -> e)
-	| TParenthesis ({ eexpr = TConst _ } as ec) ->
+	| TCall ({ eexpr = TFunction func } as ef,el) ->
+		(match follow ef.etype with
+		| TFun (_,rt) ->
+			let cf = { cf_name = ""; cf_params = []; cf_type = ef.etype; cf_public = true; cf_doc = None; cf_get = NormalAccess; cf_set = NoAccess; cf_expr = None } in
+			let inl = (try type_inline ctx cf func (mk (TConst TNull) (mk_mono()) e.epos) el rt e.epos with Error (Custom _,_) -> None) in
+			(match inl with
+			| None -> e
+			| Some e -> e)
+		| _ -> 
+			e)
+	| TParenthesis ({ eexpr = TConst _ } as ec) | TBlock [{ eexpr = TConst _ } as ec] ->
 		{ ec with epos = e.epos }
-	| _ -> 
+	| TSwitch (_,cases,_) ->
+		List.iter (fun (cl,_) ->
+			List.iter (fun e ->
+				match e.eexpr with
+				| TCall ({ eexpr = TEnumField _ },_) -> error "Not-constant enum in switch cannot be matched" e.epos
+				| _ -> ()
+			) cl
+		) cases;
+		e
+	| _ ->
 		e
 
-let reduce_expression com e =
-	reduce_loop com false e
+let reduce_expression ctx e =
+	if ctx.com.foptimize then reduce_loop ctx false e else e
+
+(* ---------------------------------------------------------------------- *)
+(* SANITIZE *)
+
+(*
+	makes sure that when an AST get generated to source code, it will not
+	generate expressions that evaluate differently. It is then necessary to
+	add parenthesises around some binary expressions when the AST does not
+	correspond to the natural operand priority order for the platform
+*)
+
+let rec sanitize e =	
+	match e.eexpr with
+	| TBinop (op,e1,e2) ->
+		let parent e = mk (TParenthesis e) e.etype e.epos in
+		let e1 = sanitize e1 in
+		let e2 = sanitize e2 in
+		let e1 = (match e1.eexpr with
+			| TBinop (op2,_,_) when Parser.swap op2 op -> parent e1
+			| _ -> e1
+		) in
+		let e2 = (match e2.eexpr with
+			| TBinop (op2,_,_) when Parser.swap op2 op -> parent e2
+			| _ -> e2
+		) in
+		{ e with eexpr = TBinop (op,e1,e2) }
+	| _ -> Type.map_expr sanitize e
+
