@@ -246,7 +246,7 @@ let rec type_module_type ctx t tparams p =
 			error (s_type_path s.t_path ^ " is not a value") p
 
 let type_type ctx tpath p =
-	type_module_type ctx (Typeload.load_type_def ctx p tpath) None p
+	type_module_type ctx (Typeload.load_type_def ctx p { tpackage = fst tpath; tname = snd tpath; tparams = []; tsub = None }) None p
 
 let get_constructor c p =
 	let rec loop c = 
@@ -319,17 +319,17 @@ let rec acc_get ctx g p =
 			loop e
 
 let field_access ctx mode f t e p =
+	let normal() = AccExpr (mk (TField (e,f.cf_name)) t p) in
 	match (match mode with MGet | MCall -> f.cf_get | MSet -> f.cf_set) with
 	| NoAccess ->
-		let normal = AccExpr (mk (TField (e,f.cf_name)) t p) in
 		(match follow e.etype with
-		| TInst (c,_) when is_parent c ctx.curclass -> normal
+		| TInst (c,_) when is_parent c ctx.curclass -> normal()
 		| TAnon a ->
 			(match !(a.a_status) with
-			| Statics c2 when ctx.curclass == c2 -> normal
-			| _ -> if ctx.untyped then normal else AccNo f.cf_name)
+			| Statics c2 when ctx.curclass == c2 -> normal()
+			| _ -> if ctx.untyped then normal() else AccNo f.cf_name)
 		| _ ->
-			if ctx.untyped then normal else AccNo f.cf_name)
+			if ctx.untyped then normal() else AccNo f.cf_name)
 	| MethodAccess false when not ctx.untyped ->
 		error "Cannot rebind this method : please use 'dynamic' before method declaration" p
 	| NormalAccess | MethodAccess _ ->
@@ -340,7 +340,10 @@ let field_access ctx mode f t e p =
 		(match mode, f.cf_set with
 		| MGet, MethodAccess _ -> AccExpr (mk (TClosure (e,f.cf_name)) t p)
 		| MGet, NoAccess | MGet, NeverAccess when (match follow t with TFun _ -> true | _ -> false) -> AccExpr (mk (TClosure (e,f.cf_name)) t p)
-		| _ -> AccExpr (mk (TField (e,f.cf_name)) t p))
+		| _ ->
+			match follow e.etype with
+			| TAnon a -> (match !(a.a_status) with EnumStatics e -> AccExpr (mk (TEnumField (e,f.cf_name)) t p) | _ -> normal())
+			| _ -> normal())
 	| CallAccess m ->
 		if m = ctx.curmethod && (match e.eexpr with TConst TThis -> true | TTypeExpr (TClassDecl c) when c == ctx.curclass -> true | _ -> false) then
 			let prefix = if Common.defined ctx.com "as3" then "$" else "" in
@@ -358,6 +361,28 @@ let field_access ctx mode f t e p =
 	| InlineAccess ->
 		AccInline (e,f,t)
 
+let using_field ctx mode e i p =
+	if mode = MSet then raise Not_found;
+	let rec loop = function
+		| [] ->
+			raise Not_found
+		| TEnumDecl _ :: l | TTypeDecl _ :: l ->
+			loop l
+		| TClassDecl c :: l ->
+			try
+				let f = PMap.find i c.cl_statics in
+				let t = field_type f in
+				(match follow t with
+				| TFun ((_,_,t0) :: args,r) ->
+					(try unify_raise ctx e.etype t0 p with Error (Unify _,_) -> raise Not_found);
+					let et = type_module_type ctx (TClassDecl c) None p in						
+					AccUsing (mk (TField (et,i)) t p,e)
+				| _ -> raise Not_found)
+			with Not_found ->
+				loop l
+	in
+	loop ctx.local_using
+
 let type_ident ctx i is_type p mode =
 	match i with
 	| "true" ->
@@ -397,7 +422,7 @@ let type_ident ctx i is_type p mode =
 		let infos = mk_infos ctx p [] in
 		let e = type_expr ctx infos true in
 		if mode = MGet then
-			AccExpr { e with etype = Typeload.load_normal_type ctx { tpackage = ["haxe"]; tname = "PosInfos"; tparams = [] } p false }
+			AccExpr { e with etype = Typeload.load_instance ctx { tpackage = ["haxe"]; tname = "PosInfos"; tparams = []; tsub = None } p false }
 		else
 			AccNo i
 	| _ ->
@@ -409,6 +434,8 @@ let type_ident ctx i is_type p mode =
 		if ctx.in_static then raise Not_found;
 		let t , f = class_field ctx.curclass i in
 		field_access ctx mode f t (mk (TConst TThis) ctx.tthis p) p
+	with Not_found -> try
+		using_field ctx mode (mk (TConst TThis) ctx.tthis p) i p
 	with Not_found -> try
 		(* static variable lookup *)
 		let f = PMap.find i ctx.curclass.cl_statics in
@@ -493,28 +520,6 @@ let type_matching ctx (enum,params) (e,p) ecases first_case =
 		invalid()
 
 let rec type_field ctx e i p mode =
-	let using_field() =
-		if mode = MSet then raise Not_found;
-		let rec loop = function
-			| [] ->
-				raise Not_found
-			| TEnumDecl _ :: l | TTypeDecl _ :: l ->
-				loop l
-			| TClassDecl c :: l ->
-				try
-					let f = PMap.find i c.cl_statics in
-					let t = field_type f in
-					(match follow t with
-					| TFun ((_,_,t0) :: args,r) ->
-						(try unify_raise ctx e.etype t0 p with Error (Unify _,_) -> raise Not_found);
-						let et = type_module_type ctx (TClassDecl c) None p in						
-						AccUsing (mk (TField (et,i)) t p,e)
-					| _ -> raise Not_found)
-				with Not_found ->
-					loop l
-		in
-		loop ctx.local_using
-	in
 	let no_field() =
 		if not ctx.untyped then display_error ctx (s_type (print_context()) e.etype ^ " has no field " ^ i) p;
 		AccExpr (mk (TField (e,i)) (mk_mono()) p)
@@ -540,7 +545,7 @@ let rec type_field ctx e i p mode =
 			if not f.cf_public && not (is_parent c ctx.curclass) && not ctx.untyped then display_error ctx ("Cannot access to private field " ^ i) p;
 			field_access ctx mode f (apply_params c.cl_types params t) e p
 		with Not_found -> try
-			using_field()
+			using_field ctx mode e i p
 		with Not_found -> try
 			loop_dyn c params
 		with Not_found ->
@@ -560,7 +565,7 @@ let rec type_field ctx e i p mode =
 			field_access ctx mode f (field_type f) e p
 		with Not_found ->
 			if is_closed a then try
-				using_field()
+				using_field ctx mode e i p
 			with Not_found ->
 				no_field()
 			else
@@ -594,8 +599,8 @@ let rec type_field ctx e i p mode =
 		ctx.opened <- x :: ctx.opened;
 		r := Some t;
 		field_access ctx mode f (field_type f) e p
-	| t ->
-		try using_field() with Not_found -> no_field()
+	| _ ->
+		try using_field ctx mode e i p with Not_found -> no_field()
 
 (*
 	We want to try unifying as an integer and apply side effects.
@@ -965,7 +970,10 @@ and type_switch ctx e cases def need_val p =
 			v
 		) el in
 		if el = [] then error "Case must match at least one expression" (pos e2);
-		let e2 = type_expr ctx ~need_val e2 in
+		let e2 = (match fst e2 with 
+			| EBlock [] -> mk (TConst TNull) ctx.api.tvoid (pos e2)
+			| _ -> type_expr ctx ~need_val e2
+		) in
 		locals();
 		unify_val e2;
 		(el,e2)
@@ -1076,7 +1084,7 @@ and type_access ctx e p mode =
 									raise (Error (Module_not_found (List.rev !path,name),p))
 								with
 									Not_found ->
-										if ctx.in_display then raise (Parser.TypePath (List.map (fun (n,_,_) -> n) (List.rev acc)));
+										if ctx.in_display then raise (Parser.TypePath (List.map (fun (n,_,_) -> n) (List.rev acc),None));
 										raise e)
 				| (_,false,_) as x :: path ->
 					loop (x :: acc) path
@@ -1338,7 +1346,7 @@ and type_expr ctx ?(need_val=true) (e,p) =
 	| ETry (e1,catches) ->
 		let e1 = type_expr ctx ~need_val e1 in
 		let catches = List.map (fun (v,t,e) ->
-			let t = Typeload.load_type ctx (pos e) t in
+			let t = Typeload.load_complex_type ctx (pos e) t in
 			let name = (match follow t with
 				| TInst ({ cl_path = path },params) | TEnum ({ e_path = path },params) ->
 					List.iter (fun pt ->
@@ -1365,7 +1373,7 @@ and type_expr ctx ?(need_val=true) (e,p) =
 	| ECall (e,el) ->
 		type_call ctx e el p
 	| ENew (t,el) ->
-		let t = Typeload.load_normal_type ctx t p true in
+		let t = Typeload.load_instance ctx t p true in
 		let el, c , params = (match follow t with
 		| TInst (c,params) ->
 			let name = (match c.cl_path with [], name -> name | x :: _ , _ -> x) in
@@ -1428,7 +1436,7 @@ and type_expr ctx ?(need_val=true) (e,p) =
 	| ECast (e, Some t) ->
 		(* // if( Std.is(tmp,T) ) tmp else throw "Class cast error" *)
 		let etmp = (EConst (Ident "tmp"),p) in
-		let t = Typeload.load_type ctx (pos e) t in
+		let t = Typeload.load_complex_type ctx (pos e) t in
 		let tname = (match follow t with
 		| TInst (_,params) | TEnum (_,params) ->
 			List.iter (fun pt ->
@@ -1455,7 +1463,7 @@ and type_expr ctx ?(need_val=true) (e,p) =
 	| EDisplay e ->
 		let old = ctx.in_display in
 		ctx.in_display <- true;
-		let e = (try type_expr ctx e with Error (Unknown_ident n,_) -> raise (Parser.TypePath [n])) in
+		let e = (try type_expr ctx e with Error (Unknown_ident n,_) -> raise (Parser.TypePath ([n],None))) in
 		ctx.in_display <- old;
 		let t = (match follow e.etype with
 			| TInst (c,params) ->
@@ -1511,7 +1519,7 @@ and type_expr ctx ?(need_val=true) (e,p) =
 		) in
 		raise (Display t)
 	| EDisplayNew t ->
-		let t = Typeload.load_normal_type ctx t p true in
+		let t = Typeload.load_instance ctx t p true in
 		(match follow t with
 		| TInst (c,params) ->
 			let f = get_constructor c p in
@@ -1651,8 +1659,7 @@ let get_type_module ctx t =
 		(* @Main, other generated classes ? *)
 		{
 			mtypes = [t];
-			mpath = t_path t;
-			mimports = [];
+			mpath = t_path t;			
 		}
 	with
 		Exit -> !mfound
@@ -1763,7 +1770,7 @@ let types ctx main excludes =
 	(match main with
 	| None -> ()
 	| Some cl ->
-		let t = Typeload.load_type_def ctx null_pos cl in
+		let t = Typeload.load_type_def ctx null_pos { tpackage = fst cl; tname = snd cl; tparams = []; tsub = None } in
 		let ft, r = (match t with
 		| TEnumDecl _ | TTypeDecl _ ->
 			error ("Invalid -main : " ^ s_type_path cl ^ " is not a class") null_pos
@@ -1802,8 +1809,7 @@ let types ctx main excludes =
 let create com =
 	let empty =	{
 		mpath = [] , "";
-		mtypes = [];
-		mimports = [];
+		mtypes = [];		
 	} in
 	let ctx = {
 		com = com;
@@ -1838,7 +1844,7 @@ let create com =
 	ctx.api.build_instance <- Codegen.build_instance ctx;
 	ctx.api.on_generate <- Codegen.on_generate ctx;
 	ctx.api.get_type_module <- get_type_module ctx;
-	ctx.api.optimize <- Optimizer.reduce_expression com;
+	ctx.api.optimize <- Optimizer.reduce_expression ctx;
 	ctx.std <- (try
 		Typeload.load_module ctx ([],"StdTypes") null_pos
 	with
